﻿/********************************************************************************/
/* 	state.d																		*/
/*------------------------------------------------------------------------------*/
/*	製作		D.K（あるふぁ～秘密基地）										*/
/*	製作開始	2008/05/22														*/
/*	MAIL		dk@red.interq.or.jp												*/
/*	URL			http://maglog.jp/alpha-secret-base/								*/
/*																				*/
/*	このソースは「やわらかライセンス」の元で配布されています。					*/
/*-更新履歴---------------------------------------------------------------------*/
/*	2008/05/22	初版															*/
/*-その他-----------------------------------------------------------------------*/
/*	なし																		*/
/********************************************************************************/

/* ライブラリのインポート *******************************************************/
private import hell2;		// dHell2ライブラリ
private import vector;
private import keypad;
private import actor;
private import clickable;
private import gamecore;

private import std.string;
private import std.math;
private import std.math2;
private import std.stream;
private import std.thread;

import config;

/* グローバル定数定義 ***********************************************************/
GameState g_gamestate = null;
const int  SCR_W = 640;
const int  SCR_H = 360;

/* load *************************************************************************/
/*	リソースの読み込み															*/
/*-引数-------------------------------------------------------------------------*/
/*	なし																		*/
/*-返り値-----------------------------------------------------------------------*/
/*	なし																		*/
/********************************************************************************/
void load()
{
	Hell_loadTexture("font","resource/font/font.bmp" , [255,255,255]);
	Hell_loadTexture("menu","resource/image/menu.bmp" , [0,255,0]);
	
	Hell_write("resource loaded.\n");
}

/* GameState ********************************************************************/
/*	ゲーム全体の状態遷移を管理するクラス										*/
/********************************************************************************/
class GameState
{
private:
	int msg;				// リクエストメッセージ
public:
	enum
	{
		MSG_NONE = 0,
		MSG_REQ_LOGO,			// ロゴ状態へ　行きたい
		MSG_REQ_TITLE,			// タイトル状態へ　行きたい
		MSG_REQ_GAME,			// ゲーム
		
		MSG_EXIT,				//アプリケーションの終了
	}
	
	void move(){}
	void draw(){}
	
	// 設定
	void setMsg(int m)
	{
		msg = m;
	}
	// 取得
	int getMsg()
	{
		return msg;
	}
}



bool loop()
{
	bool is_exit = true;	//アプリケーションの終了
	if(g_gamestate)
	{
		// メッセージ指示による状態遷移
		switch(g_gamestate.msg)
		{
			case GameState.MSG_NONE:
				break;
			case GameState.MSG_REQ_LOGO:
				g_gamestate = new LogoMain();
				break;
			case GameState.MSG_REQ_TITLE:
				g_gamestate = new TitleMain();
				break;
			case GameState.MSG_REQ_GAME:
				g_gamestate = new GameMain();
				break;
			case GameState.MSG_EXIT:
			default:
				is_exit = false;
				Hell_write("Unknown msg at loop.");
		}
	}
	else
	{
		// 初期状態
		g_gamestate = new LogoMain();
		Hell_write("Exec loop.\n");
	}
	
	g_gamestate.move();
	g_gamestate.draw();
	
	return( is_exit );
}

/* TitleMain ********************************************************************/
/*	タイトルの状態																*/
/********************************************************************************/

class TitleMain : GameState
{
public:
	int timer;
	Keypad keypad;					// キーボード/ゲームパッド取得用
	int init_flag;
	ConfigParser savedata;			// セーブデータ
	int hi_score;
	int star;
	int maxstar;
	
	this()
	{
Hell_write( "TitleMain\n" );
		setMsg( GameState.MSG_NONE );
		timer = 0;
		init_flag = 0;
		star = 0;

		Hell_playBgm( "resource/sound/title.ogg", -1, 0 );

		keypad = new Keypad();
		//セーブデータ
		hi_score = 0;
		savedata = new ConfigParser();
		try
		{
			savedata.load(CONFIG_PATH);
		}
		catch(Exception e)
		{
			savedata.set("HI_SCORE",         999999);
			savedata.write();
		}
		hi_score = savedata.getint("HI_SCORE");
		
	}
	
	void move()
	{
		if ( init_flag == 0 )
		{
			init_flag = 1;
//			Hell_loadTexture("title","resource/image/title.bmp" , [0,255,0]);
//			Hell_playBgm( "resource/sound/title.ogg", -1, 0 );
//			Hell_stopBgm( 1000 );
			Hell_loadTexture("bg","resource/image/bg.bmp" , [0,255,0]);
		}
		timer++;
		
		if(keypad.isPushEnter())
		{
			//１ボタン
			setMsg(GameState.MSG_REQ_GAME);
		}
/*
		if(keypad.isPushCancel())
		{
			//2ボタン
			savedata.set("SELECT_HOSHI",         star);
			savedata.write();

			Hell_SetGameflag( 0, 1 );	//リプレイモードON
			Hell_SetGameflag( 1, star );	//星番号
			setMsg(GameState.MSG_REQ_GAME);
		}
*/
		if(keypad.isPushLeft())
		{
		}
		else if(keypad.isPushRight())
		{
		}
	}
	
	void draw()
	{
		int min;
		int sec;
		int msec;
		char[] str;

		Hell_begin();
		resetPerspective();
		Hell_setAlpha(HELL_ALPHA_NORMAL);
	
		Hell_drawTextureEx("bg"
			, SCR_W / 2, SCR_H / 2
			, 0, 0
			, SCR_W, SCR_H );

		Hell_drawTextureEx("title"
			, SCR_W / 2, SCR_H / 2
			, 0, 0
			, SCR_W, SCR_H );


/*
		min = ( hi_score / 60 ) / 60;
		sec = ( hi_score / 60 ) % 60;
		msec = hi_score % 60;
		str = std.string.format( "%02d:%02d:%02d", min, sec, msec );
		Hell_drawFont( "HI SCORE" , 500, 2 , 1 , 255,255,255);
		Hell_drawFont( str , 500, 10 , 2 , 255,255,255);
*/
		if (( Hell_getCount() % 120 ) > 60 ) 
		{
			Hell_drawFont( "PURESS Z OR 1 BUTTON TO START"  , 100, 300 , 2 , 255,255,255);
		}

//		Hell_drawFont("HOGE" , 240, 160 , 4 , 255,255,255);
		Hell_drawFont("ALPHA SECRET BASE 2017" , 280, 360-20 , 2 , 255,255,255);
	}
}

/* TitleMain ********************************************************************/
/*	ロゴの状態																	*/
/********************************************************************************/

class LogoMain : GameState
{
public:
	int timer;
	Keypad keypad;					// キーボード/ゲームパッド取得用
	int init_flag;
	
	this()
	{
		setMsg( GameState.MSG_NONE );
		timer = 0;
		init_flag = 0;
		
		keypad = new Keypad();
//		menu = new ClickablePool();
//		menu.add("START" , new Vec3(280 , 400 , 0) , new Vec3(96 , 32 , 0));
		
	}
	
	void move()
	{
		if ( init_flag == 0 )
		{
			init_flag = 1;
			Hell_loadTexture("logo","resource/image/logo.bmp" , [0,255,0]);
			Hell_loadTexture("title","resource/image/title.bmp" , [0,255,0]);
			Hell_loadTexture("bg","resource/image/bg.bmp" , [0,255,0]);
		}
		timer++;
		
		if ( ( keypad.isPushEnter() ) || ( keypad.isPushCancel() ) || ( timer > 120 ) )
		{
//			Hell_playBgm( "resource/sound/99.ogg" );
			setMsg(GameState.MSG_REQ_TITLE);
			//１ボタン
		}
	}
	
	void draw()
	{
		Hell_begin();
		resetPerspective();
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		
		if ( timer < 120 )
		{
			Hell_drawTextureEx("logo"
				, SCR_W / 2, SCR_H / 2
				, 0, 0
				, 320, 240 );
		}
	}
}

